<?php
/**
 * montonio/module-shipping
 *
 * @category   montonio
 * @package    module-shipping
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */

namespace Montonio\Shipping\Controller\Adminhtml\Configuration;

use Montonio\Shipping\Model\Shipping\ShippingsConfigProvider;
use Montonio\Shipping\Gateway\Shippings\Config\Config;
use Montonio\Shipping\Gateway\Shippings\MontonioSDK;
use Montonio\Shipping\Model\ResourceModel\ShippingPoints\Collection as ShippingPoinsCollection;
use Montonio\Shipping\Model\ResourceModel\ShippingPoints\CollectionFactory as ShippingPointsCollectionFactory;
use Montonio\Shipping\Model\ShippingPoints;
use Montonio\Shipping\Model\ShippingPointsFactory;
use InvalidArgumentException;
use Magento\Backend\App\Action;
use Magento\Framework\Controller\Result\Redirect;
use Magento\Framework\Controller\ResultFactory;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\UrlInterface;

class Connection extends Action
{
    /**
     * @var MontonioSDK
     */
    protected $montonioSDK;

    /**
     * @var ShippingPoinsCollectionFactory
     */
    protected $shippingPointsCollection;

    /**
     * ShippingPointsFactory
     * @var null|ShippingPointsFactory
     */
    protected $shippingPointsFactory = null;

    /**
     * @var UrlInterface
     */
    protected $urlInterface;
    /**
     * Construct function.
     *
     * @param Action\Context $context
     * @param MontonioSDK $montonioSDK
     * @param ShippingPointsCollectionFactory $shippingPointsCollection
     * @param UrlInterface $urlInterface
     * @param ShippingPointsFactory $shippingPointsFactory

     */
    public function __construct(
        Action\Context $context,
        MontonioSDK $montonioSDK,
        ShippingPointsCollectionFactory $shippingPointsCollection,
        UrlInterface $urlInterface,
        ShippingPointsFactory $shippingPointsFactory)
    {
        parent::__construct($context);
        $this->montonioSDK = $montonioSDK;
        $this->shippingPointsCollection = $shippingPointsCollection;
        $this->shippingPointsFactory = $shippingPointsFactory;
        $this->urlInterface = $urlInterface;
    }

    /**
     * @inheritdoc
     * @throws InvalidArgumentException
     */
    public function execute()
    {
        try {
            $url = $this->urlInterface->getBaseUrl().'montonio/shipping/webhook';
            $this->montonioSDK->registerTrackingWebhook($url);

            $methods = $this->retrieveApiData();
            $this->setData(sprintf(Config::DEFAULT_PATH_PATTERN, ShippingsConfigProvider::CODE, 'montonio_methods/available_methods'), json_encode($methods));
            $this->messageManager->addSuccessMessage(__('Update data succeeded.'));

        } catch (\Exception $e) {
            //silence
            $this->silence();
            $this->messageManager->addErrorMessage(__('Montonio Shipping was unable to sync pickup points in the background. Please make sure your Montonio API credentials are correct.'));

        }


        /** @var Redirect $result */
        $result = $this->resultFactory->create(ResultFactory::TYPE_REDIRECT);
        return $result->setUrl($this->_redirect->getRefererUrl());
    }


    /**
     * Retrieve payment methods API data.
     *
     * @return array
     */
    private function retrieveApiData(): array
    {
        $output = [];
        try {
            $methods = $this->montonioSDK->getPickupPoints();

            $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
            $config = $objectManager->create('Montonio\Shipping\Gateway\Shippings\Config\Config');
            $oldPoints = $config->getValue('montonio_methods/available_methods');
            $oldPoints = (array)json_decode($oldPoints);


            foreach ($methods as $country => $methodsData) {

                $sort = ShippingsConfigProvider::DEFAULT_SORT_ORDER;
                foreach ($methodsData->providers as $providerName => $provider) {
                    $courier = false;

                    foreach ($provider as $type => $points) {
                        if(strpos($type, 'courier') !== false)
                            $courier = true;

                        $items = [];
                        foreach ($points  as $method) {
                            $items[] = [
                                'point_uuid' => $method->uuid,
                                'point_shipping_code' => $country . '_' . $providerName . '_' . $type,
                                'point_name' => $method->name,
                                'point_region' => $method->region,
                                'point_locality' => $method->locality,
                            ];
                        }

                        try {
                            $this->updateShippingPoints($country . '_' . $providerName . '_' . $type, $items);

                            $output[$country . '_' . $providerName . '_' . $type] = [
                                'method_country' => $country,
                                'method_enabled' => isset($oldPoints[$country . '_' . $providerName . '_' . $type]->method_enabled) ? $oldPoints[$country . '_' . $providerName . '_' . $type]->method_enabled : 0,
                                'method_title' => isset($oldPoints[$country . '_' . $providerName . '_' . $type]->method_title) ? $oldPoints[$country . '_' . $providerName . '_' . $type]->method_title : $providerName,
                                'method_name' => isset($oldPoints[$country . '_' . $providerName . '_' . $type]->method_name) ? $oldPoints[$country . '_' . $providerName . '_' . $type]->method_name : $providerName . ' ' .$type,
                                'method_provider' => $providerName,
                                'method_type' => $type,
                                'method_price' => isset($oldPoints[$country . '_' . $providerName . '_' . $type]->method_price) ? $oldPoints[$country . '_' . $providerName . '_' . $type]->method_price : 0,
                                'method_free_shipping_enabled' => isset($oldPoints[$country . '_' . $providerName . '_' . $type]->method_free_shipping_enabled) ? $oldPoints[$country . '_' . $providerName . '_' . $type]->method_free_shipping_enabled : 0,
                                'method_free_shipping_price' => isset($oldPoints[$country . '_' . $providerName . '_' . $type]->method_free_shipping_price) ? $oldPoints[$country . '_' . $providerName . '_' . $type]->method_free_shipping_price : 0,
                                'method_package_measurements_check_enabled' => isset($oldPoints[$country . '_' . $providerName . '_' . $type]->method_package_measurements_check_enabled) ? $oldPoints[$country . '_' . $providerName . '_' . $type]->method_package_measurements_check_enabled : 0,
                                'method_package_maximum_weight' => isset($oldPoints[$country . '_' . $providerName . '_' . $type]->method_package_maximum_weight) ? $oldPoints[$country . '_' . $providerName . '_' . $type]->method_package_maximum_weight : 0,
                                'method_hide_no_measurements_enabled' => isset($oldPoints[$country . '_' . $providerName . '_' . $type]->method_hide_no_measurements_enabled) ? $oldPoints[$country . '_' . $providerName . '_' . $type]->method_hide_no_measurements_enabled : 0,
                                'sort_order' => isset($oldPoints[$country . '_' . $providerName . '_' . $type]->sort_order) ? $oldPoints[$country . '_' . $providerName . '_' . $type]->sort_order : $sort
                            ];

                        } catch (\Exception $e) {
                            //silence
                            $this->silence();
                        }
                        $sort = $sort + 10;
                    }
                    if(!$courier) {
                        $type = 'courier';
                        $output[$country . '_' . $providerName . '_' . $type] = [
                            'method_country' => $country,
                            'method_enabled' => isset($oldPoints[$country . '_' . $providerName . '_' . $type]->method_enabled) ? $oldPoints[$country . '_' . $providerName . '_' . $type]->method_enabled : 0,
                            'method_title' => isset($oldPoints[$country . '_' . $providerName . '_' . $type]->method_title) ? $oldPoints[$country . '_' . $providerName . '_' . $type]->method_title : $providerName,
                            'method_name' => isset($oldPoints[$country . '_' . $providerName . '_' . $type]->method_name) ? $oldPoints[$country . '_' . $providerName . '_' . $type]->method_name : $providerName . ' ' . $type,
                            'method_provider' => $providerName,
                            'method_type' => $type,
                            'method_price' => isset($oldPoints[$country . '_' . $providerName . '_' . $type]->method_price) ? $oldPoints[$country . '_' . $providerName . '_' . $type]->method_price : 0,
                            'method_free_shipping_enabled' => isset($oldPoints[$country . '_' . $providerName . '_' . $type]->method_free_shipping_enabled) ? $oldPoints[$country . '_' . $providerName . '_' . $type]->method_free_shipping_enabled : 0,
                            'method_free_shipping_price' => isset($oldPoints[$country . '_' . $providerName . '_' . $type]->method_free_shipping_price) ? $oldPoints[$country . '_' . $providerName . '_' . $type]->method_free_shipping_price : 0,
                            'method_package_measurements_check_enabled' => isset($oldPoints[$country . '_' . $providerName . '_' . $type]->method_package_measurements_check_enabled) ? $oldPoints[$country . '_' . $providerName . '_' . $type]->method_package_measurements_check_enabled : 0,
                            'method_package_maximum_weight' => isset($oldPoints[$country . '_' . $providerName . '_' . $type]->method_package_maximum_weight) ? $oldPoints[$country . '_' . $providerName . '_' . $type]->method_package_maximum_weight : 0,
                            'method_hide_no_measurements_enabled' => isset($oldPoints[$country . '_' . $providerName . '_' . $type]->method_hide_no_measurements_enabled) ? $oldPoints[$country . '_' . $providerName . '_' . $type]->method_hide_no_measurements_enabled : 0,
                            'sort_order' => isset($oldPoints[$country . '_' . $providerName . '_' . $type]->sort_order) ? $oldPoints[$country . '_' . $providerName . '_' . $type]->sort_order : $sort
                        ];
                    }
                }
            }
        } catch (\Exception $e) {
            //silence
            $this->silence();
        }
        return $output;
    }

    public function setData($path, $value) {
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $configWriter = $objectManager->create('Magento\Framework\App\Config\Storage\WriterInterface');
        $configWriter->save($path, $value, $scope = ScopeConfigInterface::SCOPE_TYPE_DEFAULT, $scopeId = 0);
    }

    public function updateShippingPoints($shipping_code, $points)
    {
        $oldPoints = array();
        $parcelPoints = $this->shippingPointsCollection->create();
        $parcelPoints->addFieldToFilter('shipping_code', $shipping_code)->addFieldToSelect('*')->load();

        $collection = $parcelPoints->getItems();
        foreach($collection as $item) {
            $oldPoints[$item->getUuid()] = $item->getUuid();
        }

        foreach ($points as $place) {
            $parcelPoint = $this->shippingPointsFactory->create();
            try {
                $parcelPoint->load($place['point_uuid']);
                if(isset($oldPoints[$parcelPoint->getUuid()]))
                    unset($oldPoints[$parcelPoint->getUuid()]);
            }
            catch (\Exception $e) {
                //silence
                $this->silence();
            }

            if(!$parcelPoint || !$parcelPoint->getUuid()){
                $parcelPoint->setUuid($place['point_uuid']);
            }

            $parcelPoint->setShippingCode($place['point_shipping_code']);
            $parcelPoint->setName($place['point_name']);
            $parcelPoint->setRegion($place['point_region']);
            $parcelPoint->setLocality( $place['point_locality']);
            try {
                $parcelPoint->save();
            }
            catch (\Exception $e) {
            }
        }

        foreach($oldPoints as $point) {
            $parcelPoint = $this->shippingPointsFactory->create();
            $parcelPoint->load($point);
            $parcelPoint->delete();
        }
    }

    /**
     * Do nothing
     * @return void
     */
    public function silence() : void
    {
    }

}
