<?php
/**
 * montonio/module-shipping
 *
 * @category   montonio
 * @package    module-shipping
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */

namespace Montonio\Shipping\Controller\Shipping;

use Throwable;
use Magento\Framework\App\CsrfAwareActionInterface;
use Magento\Framework\App\Request\InvalidRequestException;
use Magento\Framework\App\RequestInterface;

class Webhook extends \Magento\Framework\App\Action\Action implements CsrfAwareActionInterface
{

    /**
     * @var Http
     */

    protected $request;
    /**
     * @var Config
     */

    private $config;
    /**
     * @var LoggerInterface
     */

    private $logger;

    /**
     * @var OrderFactory
     */
    protected $orderFactory;

    /**
     * @var ShipmentNotifier
     */

    private $shipmentNotifier;
    /**
     * @var Order
     */

    private $orderConverter;

    /**
     * @var ShipmentTrackInterfaceFactory
     */
    private $trackFactory;

    /**
     * @param \Magento\Framework\App\Action\Context $context
     * @param \Montonio\Shipping\Gateway\Shippings\Config\Config $config
     * @param \Magento\Framework\App\Request\Http $request
     * @param \Magento\Shipping\Model\ShipmentNotifier $shipmentNotifier
     * @param \Magento\Sales\Model\Convert\Order $orderConverter
     * @param \Magento\Sales\Api\Data\ShipmentTrackInterfaceFactory $trackFactory
     * @param \Psr\Log\LoggerInterface $logger
     */
    public function __construct(
        \Magento\Framework\App\Action\Context $context,
        \Magento\Framework\App\Request\Http $request,
        \Magento\Sales\Model\OrderFactory $orderFactory,
        \Magento\Shipping\Model\ShipmentNotifier $shipmentNotifier,
        \Magento\Sales\Model\Convert\Order $orderConverter,
        \Magento\Sales\Api\Data\ShipmentTrackInterfaceFactory $trackFactory,
        \Montonio\Shipping\Gateway\Shippings\Config\Config $config,
        \Psr\Log\LoggerInterface $logger
    ) {

        $this->request = $request;
        $this->config = $config;
        $this->orderFactory = $orderFactory;
        $this->logger = $logger;
        $this->orderConverter = $orderConverter;
        $this->shipmentNotifier = $shipmentNotifier;
        $this->trackFactory = $trackFactory;

        parent::__construct($context);
    }

    /**
     * @inheritDoc
     */
    public function createCsrfValidationException(
        RequestInterface $request
    ): ?InvalidRequestException {
        return null;
    }

    /**
     * @inheritDoc
     */
    public function validateForCsrf(RequestInterface $request): ?bool
    {
        return true;
    }


    /**
     * @inheritdoc
     */
    public function execute()
    {

        if (!$this->request->getParam('webhook_token')) {
            http_response_code(400);
            return;
        }
        $token = $this->request->getParam('webhook_token');
        $environment = $this->config->getValue('environment');

        try {
            $response = $this->config->decodeSignature($token, $this->config->getValue($environment.'_private_key'));
        } catch (\Exception $exception) {
            http_response_code(401);
            return;
        }
        $webhookMessage = $response->message;

        switch($webhookMessage) {
            case 'shipment_created':
                $this->handleShipmentCreatedWebhook($response);
                break;
            case 'label_created':
                $this->handleLabelCreatedWebhook($response);
                break;
            default:
                break;
        }

    }

    protected function handleShipmentCreatedWebhook($response)
    {
        $orderId = $response->merchant_reference;
        $orderModel = $this->orderFactory->create();
        $order = $orderModel->loadByIncrementId($orderId);

        if (!$order) {
            http_response_code(409);
            return;
        }

        $trackingCodesText = $this->processTrackingCodes($response->tracking_numbers);
        $trackingLinksText = $this->processTrackingLinks($response->tracking_numbers);

        $orderNote = __('Montonio Shipping: shipment created.');
        if ($trackingCodesText) {
            $orderNote .= '<br>'.__('Tracking codes: ')  . $trackingLinksText;
        }

        if($trackingCodesText) {

            try {
                // save order
                $order->addStatusHistoryComment($orderNote)->setIsCustomerNotified(null);
                $order->setData('montonio_tracking_info', ($this->config->getValue('tracking_code_text') ? $this->config->getValue('tracking_code_text') : __('Track your shipment:')) . ' ' .$trackingLinksText);
                $order->save();

            } catch (\Exception $e) {
                $this->logger->error('Creating shipment for order FAILED', [
                    'exception_message' => $e->getMessage()
                ]);
            }
        }

        http_response_code(200);
    }

    protected function handleLabelCreatedWebhook($response)
    {
        $orderId = $response->merchant_reference;
        $orderModel = $this->orderFactory->create();
        $order = $orderModel->loadByIncrementId($orderId);
        $createShipment = $this->config->getValue('create_shipment');

        if (!$order) {
            http_response_code(409);
            return;
        }

        $trackingCodesText = $this->processTrackingCodes($response->tracking_numbers);
        $trackingLinksText = $this->processTrackingLinks($response->tracking_numbers);

        $orderNote = __('Montonio Shipping: Label printed.');
        if ($trackingCodesText) {
            $orderNote .= '<br>'.__('Tracking codes: ')  . $trackingLinksText;
        }

        if($order->canShip() && $trackingCodesText && $createShipment) {

            // initialize the order shipment object
            $shipment = $this->orderConverter->toShipment($order);

            // loop through order items
            foreach ($order->getAllItems() as $orderItem) {

                if (!$orderItem->getQtyToShip() || $orderItem->getIsVirtual()) {
                    continue;
                }

                $shipQty = $orderItem->getQtyToShip();
                $shipmentItem = $this->orderConverter->itemToShipmentItem($orderItem)->setQty($shipQty);
                $shipment->addItem($shipmentItem);
            }

            // register shipment
            $shipment->register();

            try {
                // save created shipment
                $track = $this->trackFactory->create()->setNumber(
                    $trackingCodesText
                )->setCarrierCode(
                    'montonio_shipping'
                )->setTitle(
                    'Montonio tracking info'
                );
                $shipment->addTrack($track);

                // save order
                $order->addStatusHistoryComment($orderNote)->setIsCustomerNotified(null);
                $order->save();

                // send email
                $this->shipmentNotifier->notify($shipment);
                $shipment->save();

            } catch (\Exception $e) {
                $this->logger->error('Creating shipment for order FAILED', [
                    'exception_message' => $e->getMessage()
                ]);
            }
        } else {
            try {
                // save order
                $order->addStatusHistoryComment($orderNote)->setIsCustomerNotified(null);
                $order->save();

            } catch (\Exception $e) {
                $this->logger->error('Save order FAILED', [
                    'exception_message' => $e->getMessage()
                ]);
            }
        }

        http_response_code(200);
    }

    protected function processTrackingLinks($trackingCodes)
    {
        $trackingCodesText = '';
        foreach ($trackingCodes as $trackingCode) {
            $trackingCodesText .= '
            <a target="_blank" href=' . $trackingCode->link . '>'
                . $trackingCode->code .
                '</a><br>';
        }
        return $trackingCodesText;
    }

    protected function processTrackingCodes($trackingCodes)
    {
        $trackingCodesText = '';
        foreach ($trackingCodes as $trackingCode) {
            $trackingCodesText .= $trackingCode->code. ' ' ."\n";
        }
        return $trackingCodesText;
    }
}
