<?php
/**
 * montonio/module-shipping
 *
 * @category   montonio
 * @package    module-shipping
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */
declare(strict_types=1);
namespace Montonio\Shipping\Gateway\Shippings\Config;

use \Firebase\JWT\JWT;
use \Firebase\JWT\Key;
use Magento\Framework\App\ObjectManager;
use Magento\Store\Model\ScopeInterface;

/**
 * @inheritdoc
 */
class Config
{

    const DEFAULT_PATH_PATTERN = 'carriers/%s/%s';


    /**
     *
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Magento\Framework\Escaper $escaper
     * @param \Magento\Framework\Serialize\Serializer\Json $json
     * @param string $methodCode
     * @param string $pathPattern
     */
    public function __construct(
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Framework\Escaper $escaper,
        \Magento\Framework\Serialize\Serializer\Json $json,
        \Psr\Log\LoggerInterface $logger,
        $methodCode = null,
        $pathPattern = self::DEFAULT_PATH_PATTERN
    ) {
        $this->scopeConfig = $scopeConfig;
        $this->escaper = $escaper;
        $this->json = $json;
        $this->logger = $logger;

        //if we DI this class directly to our other components we need this to be initiated
        //in all other case it is initiated trough di.xml
        if (!$methodCode) {
            $methodCode = \Montonio\Shipping\Model\Shipping\ShippingsConfigProvider::CODE;
        }

        $this->methodCode = $methodCode;
        $this->pathPattern = $pathPattern;
    }

    /**
     * Retrieve information from shipping configuration
     *
     * @param string $field
     * @param int|null $storeId
     *
     * @return mixed
     */
    public function getValue($field, $storeId = null)
    {
        if ($this->methodCode === null || $this->pathPattern === null) {
            return null;
        }

        return $this->scopeConfig->getValue(
            sprintf($this->pathPattern, $this->methodCode, $field),
            ScopeInterface::SCOPE_STORE,
            $storeId
        );
    }

    /**
     * @return bool
     */
    public function isActive($storeId = null)
    {
        return $this->getValue('active', $storeId);
    }

    /**
     *
     * @return string
     */
    public function getCode()
    {
        return $this->methodCode;
    }


    /**
     * Retrieve title display option.
     *
     * @return string
     */
    public function getTitleDisplayOption(): string
    {
        return $this->escaper->escapeHtml((string)$this->getValue('title_display_option'));
    }


    /**
     * Retrieve assoc array of checkout configuration
     *
     * @return array
     */
    public function getConfig()
    {
        //we do not use this and return empty array
        return [];
    }

    /**
     *
     * @param mixed $data
     * @param string $key
     * @return string
     */
    public function generateSignature($data, $key, $jsonEncode = false)
    {
        if ($jsonEncode) {
            $data = json_encode($data);
        }
        return JWT::encode($data, $key, 'HS256');
    }

    /**
     *
     * @param string $data
     * @param string $key
     * @return array|string
     */
    public function decodeSignature($data, $key)
    {
        return JWT::decode($data, new Key($key, 'HS256'));
    }

    /**
     * @param string $country
     * @param string $country

     */
    public function getAvailableMethods($country = '', $storeId = null)
    {

        $methods = [];
        try {
            $values = $this->json->unserialize($this->getValue('montonio_methods/available_methods', $storeId));
            $methods = array_flip(array_keys($values));

            foreach ($methods as $key => $value) {
                if (is_array($values[$key])) {
                    $methods[$key] = $values[$key];
                }

                if (is_array($values[$key]) && !$values[$key]['method_enabled']) {
                    unset($methods[$key]);
                }

                if ($country && is_array($values[$key]) && $values[$key]['method_country'] != $country) {
                    unset($methods[$key]);
                }
            }
        } catch (\Exception $e) {
            $this->logger->critical($e);
        }

        return $methods;
    }

    /**
     * @param string $country
     * @param string $country

     */
    public function getAllMethods()
    {

        $methods = [];
        try {
            $values = $this->json->unserialize($this->getValue('montonio_methods/available_methods'));
            $methods = array_flip(array_keys($values));

            foreach ($methods as $key => $value) {
                if (is_array($values[$key])) {
                    $methods[$key] = $values[$key];
                }

            }
        } catch (\Exception $e) {
            $this->logger->critical($e);
        }

        return $methods;
    }

    /**
     * Return appropriate multiplier to convert to kg
     * @return float|int
     */
    public function getWeightMultiplier()
    {
        $weightUnit = $this->scopeConfig->getValue(
            'general/locale/weight_unit',
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );

        switch ($weightUnit) {
            case 'kgs':
                return 1;
            case 'g':
                return 0.001;
            case 'lbs':
                return 0.45;
            case 'oz':
                return 0.028;
            default:
                // Unknown unit, treat as missing measurements
                return 0;
        }
    }

}
