<?php
/**
 * montonio/module-shipping
 *
 * @category   montonio
 * @package    module-shipping
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */
namespace Montonio\Shipping\Gateway\Shippings;

/**
 * Class MontonioSDK for requesting available methods from gateway
 */
class MontonioSDK
{
    private $config;
    private $timezoneInterface;
    private $logger;

    /**
     *
     * @param \Montonio\Shipping\Gateway\Shippings\Config\Config $config
     * @param \Magento\Framework\Stdlib\DateTime\TimezoneInterface $timezoneInterface
     * @param \Psr\Log\LoggerInterface $logger
     */
    public function __construct(
        \Montonio\Shipping\Gateway\Shippings\Config\Config $config,
        \Magento\Framework\Stdlib\DateTime\TimezoneInterface $timezoneInterface,
        \Psr\Log\LoggerInterface $logger
    ) {
        $this->config = $config;
        $this->timezoneInterface = $timezoneInterface;
        $this->logger = $logger;
    }

    protected function getBearerToken()
    {
        $time = $this->timezoneInterface->date()->format('U');
        $environment = $this->config->getValue('environment');
        $data = [
            'access_key' => $this->config->getValue($environment.'_public_key'),
            'iat'        => $time,
            'exp'        => $time + (60 * 10)
        ];
        $bearerJwtToken = $this->config->generateSignature(
            $data,
            $this->config->getValue($environment.'_private_key')
        );

        return $bearerJwtToken;
    }

    /**
     * @return array
     */
    public function getPickupPoints()
    {
        $route = '/pickup-points';
        $options = array(
            'http' => array(
                'header'  => "Content-Type: application/json\r\n" .
                    "Authorization: Bearer " . $this->getBearerToken(),
                'method' => 'GET'
            )
        );
        return $this->apiRequest($route, $options);
    }

    /**
     * @param $data
     * @param bool $async
     * @return array
     */
    public function postShipment($data, $async = false)
    {
        $route = $async ? '/shipments/create-async' : '/shipments';
        $options = array(
            'http' => array(
                'header'  => "Content-Type: application/json\r\n" .
                    "Authorization: Bearer " . $this->getBearerToken(),
                'method' => 'POST',
                'content' => json_encode($data)
            )
        );


        return $this->apiRequest($route, $options);
    }

    /**
     * @param $url
     * @return array
     */
    public function registerTrackingWebhook($url)
    {
        //$route = '/stores/register-tracking-webhook';
        $route = '/stores/store-webhook';

        $options = array(
            'http' => array(
                'header' => "Content-Type: application/json\r\n" .
                    "Authorization: Bearer " . $this->getBearerToken(),
                'method' => 'PATCH',
                'content' => json_encode(array(
                    'tracking_webhook_url' => $url
                ))
            )
        );

        return $this->apiRequest($route, $options);
    }

    public function createLabels($data)
    {
        $route = '/shipments/label-from-store';
        $options = array(
            'http' => array(
                'header' => "Content-Type: application/json\r\n" .
                    "Authorization: Bearer " . $this->getBearerToken(),
                'method' => 'POST',
                'content' => json_encode($data)
            )
        );

        return $this->ApiRequest($route, $options);
    }


    /**
     * Function for making API calls with file_get_contents
     * @param Array Context Options
     * @return Array Array containing status and json_decode response
     */
    protected function apiRequest($apiEndpoint, $options)
    {
        //$writer = new \Laminas\Log\Writer\Stream(BP . '/var/log/montonio_shipping.log');
        //$logger = new \Laminas\Log\Logger();
        //$logger->addWriter($writer);
        //$logger->info('apiRequest');

        $environment = $this->config->getValue('environment');
        $apiUrl = $this->config->getValue($environment.'_api_url').$apiEndpoint;
        //$logger->info('$apiUrl: ' .$apiUrl);
        //$logger->info('$options: ' .print_r($options, true));

        try {
            $context = stream_context_create($options);
            $result = file_get_contents($apiUrl, false, $context);
            //$logger->info('$result: ' .print_r(json_decode($result), true));

            // error handling
            if ($result === FALSE) {
                $message = __('SDK request status False');
                throw new \Magento\Framework\Exception\LocalizedException($message);

            } else {
                return json_decode($result);
            }
        }
        catch (\Exception $e) {
            $this->logger->critical($e);
            throw new \Magento\Framework\Exception\LocalizedException(__($e->getMessage()));
        }

    }
}
