<?php
/**
 * montonio/module-shipping
 *
 * @category   montonio
 * @package    module-shipping
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */

namespace Montonio\Shipping\Model\Carrier;

use Magento\Quote\Model\Quote\Address\RateRequest;

class MontonioShipping extends \Magento\Shipping\Model\Carrier\AbstractCarrier
    implements \Magento\Shipping\Model\Carrier\CarrierInterface
{
    const NO_MONTONIO_SHIPPING_PRODUCT_ATTRIBUTE = 'nomontonioshipping';

    const CARRIER_CODE = 'montonio_shipping';

    /**
     * @var string
     */
    protected $_code = self::CARRIER_CODE;
    private $denyOptions = array();

    protected $service;


    protected $rateResult;

    protected $rateResultFactory;
    protected $rateMethodFactory;

    protected $config;
    protected $assetRepo;
    protected $productRepository;
    protected $attributeRepository;

    protected $escaper;


    /**
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Magento\Quote\Model\Quote\Address\RateResult\ErrorFactory $rateErrorFactory
     * @param \Psr\Log\LoggerInterface $logger
     * @param \Montonio\Shipping\Gateway\Shippings\Config\Config $config
     * @param \Magento\Shipping\Model\Rate\ResultFactory $rateResultFactory
     * @param \Magento\Quote\Model\Quote\Address\RateResult\MethodFactory $rateMethodFactory
     * @param \Magento\Shipping\Model\Rate\Result $rateResult
     * @param \Magento\Framework\View\Asset\Repository $assetRepo
     * @param \Magento\Catalog\Model\ProductRepository $productRepository
     * @param \Magento\Catalog\Api\ProductAttributeRepositoryInterface  $attributeRepository
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Quote\Model\Quote\Address\RateResult\ErrorFactory $rateErrorFactory,
        \Psr\Log\LoggerInterface $logger,
        \Montonio\Shipping\Gateway\Shippings\Config\Config $config,
        \Magento\Shipping\Model\Rate\ResultFactory $rateResultFactory,
        \Magento\Quote\Model\Quote\Address\RateResult\MethodFactory $rateMethodFactory,
        \Magento\Shipping\Model\Rate\Result $rateResult,
        \Magento\Framework\View\Asset\Repository $assetRepo,
        \Magento\Catalog\Model\ProductRepository $productRepository,
        \Magento\Catalog\Api\ProductAttributeRepositoryInterface $attributeRepository,
        \Magento\Framework\Escaper $escaper,
        array $data = []
    ) {
        $this->rateResultFactory = $rateResultFactory;
        $this->rateMethodFactory = $rateMethodFactory;
        $this->rateErrorFactory = $rateErrorFactory;
        $this->assetRepo = $assetRepo;
        $this->escaper = $escaper;
        $this->rateResult = $rateResult;
        $this->productRepository = $productRepository;
        $this->attributeRepository = $attributeRepository;

        $this->config = $config;

        $this->_initAttributeOptions(self::NO_MONTONIO_SHIPPING_PRODUCT_ATTRIBUTE);

        parent::__construct($scopeConfig, $rateErrorFactory, $logger, $data);
    }

    /**
     * @return array
     */
    public function getAllowedMethods()
    {
        $methods = [];
        $montonioMethods = $this->config->getAvailableMethods();
        foreach($montonioMethods as $key => $method) {
            $methods[$key] = $method['method_title'];
        }
        return $methods;
    }

    /**
     * @param RateRequest $request
     * @return bool|Result
     */
    public function collectRates(RateRequest $request)
    {
        $hideMethod = false;
        $noMontonioShipping = [];

        if (!$this->config->isActive()) {
            return false;
        }
        $countryId = $request->getDestCountryId();
        $storeId = $request->getStoreId();

        $montonioMethods = $this->config->getAvailableMethods($countryId, $storeId);
        $result = $this->rateResultFactory->create();

        $items = $request->getAllItems();
        $weight = 0;

        // check for excluded products
        $isDenyCheckEnabled = $this->config->getValue('deny_check_enable');

        foreach($items as $item) {
            $weight += $item->getWeight();
            if($item->getWeight() <= 0)
                $hideMethod = true;

            $product = $this->productRepository->getById($item->getProduct()->getId());

            $data = $product->getData(self::NO_MONTONIO_SHIPPING_PRODUCT_ATTRIBUTE);
            if ($data) {
                $noMontonioShipping = array_merge($noMontonioShipping, array_values(explode(',', $data)));
            }
        }


        foreach($montonioMethods as $key => $method) {
            $shippingPrice = $method['method_price'];
            $isMeasurementsCheckEnabled = $method['method_package_measurements_check_enabled'];

            if ($isMeasurementsCheckEnabled) {
                $maximumWeight = $method['method_package_maximum_weight'];
                if ($weight > $maximumWeight)
                    continue;
            }

            $isHideNoMeasurementsCheckEnabled = $method['method_hide_no_measurements_enabled'];

            if ($isHideNoMeasurementsCheckEnabled) {
                if ($hideMethod)
                    continue;
            }

            if ($isDenyCheckEnabled) {
                if(isset($this->denyOptions[$key]) && in_array($this->denyOptions[$key], $noMontonioShipping))
                    continue;
            }

/*
            $titleDisplay = $this->config->getTitleDisplayOption();
            if ($titleDisplay == 'only_logo') {
                $title = '<img src="' . $this->getImageFullPath($method) . '" alt="' . $method['method_provider'] . '" class="montonio-shipping-image">';
            } elseif ('logo_title') {
                $title = '<img src="' . $this->getImageFullPath($method) . '" alt="' . $method['method_provider'] . '" class="montonio-shipping-image">' . ' ' . $method['method_provider'];
            }
            else {
                $title = $method['method_provider'];
            }
*/
            $isFreeShippingEnabled = $method['method_free_shipping_enabled'];
            $packageValue = $request->getPackageValueWithDiscount();
            if (empty($shippingPrice) || !is_numeric($shippingPrice) || $shippingPrice < 0) {
                $shippingPrice = '0.00';
            }

            if ($isFreeShippingEnabled && $packageValue >= $method['method_free_shipping_price']) {
                $shippingPrice = '0.00';
            }
            $shippingMethod = $this->rateMethodFactory->create();

            $shippingMethod->setCarrier($this->_code);
            $shippingMethod->setCarrierTitle($method['method_title']);
            $shippingMethod->setMethod($key);
            $shippingMethod->setMethodTitle($method['method_name']);
            $shippingMethod->setPrice($shippingPrice);
            $shippingMethod->setCost($shippingPrice);

            $result->append($shippingMethod);
        }

        return $result;
    }

    /**
     * @param  $method
     * @return string
     */
    public function getImageFullPath($method)
    {
        $fileId = 'Montonio_Shipping::images/'.$method['method_provider'].'.png';

        $params = [
            'area' => 'frontend' //for admin area its backend
        ];

        $asset = $this->assetRepo->createAsset($fileId, $params);

        $imageFullPath = null;
        try {
            $imageFullPath = $asset->getSourceUrl();
        } catch (Exception $exception) {
        }

        return $imageFullPath;
    }


    private function _initAttributeOptions($attrCode = '')
    {
        if(empty($this->denyOptions)){
            $this->denyOptions = array();
            if($attr = $this->_getAttribute($attrCode)){
                $options = $attr->setStoreId(0)->getOptions();
                foreach($options as $option){
                    $this->denyOptions[$option['label']] = $option['value'];
                }
            }
        }
    }

    /**
     * Get Magento Attribute Object
     */
    private function _getAttribute($attrCode)
    {
        try{
            $attr = $this->attributeRepository->get($attrCode);
            return $attr;

        }catch(\Magento\Framework\Exception\NoSuchEntityException $e){
            // do nothing
        }

        return false;
    }
}
